//---------------------------------------------------------------------------
// File: TMainForm.h, TDx_Draw_Library, Example4 - Cooperative Levels
//
// Date: 1st February, 2002
// Author: Darren John Dwyer
// Copyright: (c) 2002 Darren John Dwyer, All Rights Reserved.
//
// Description:
//
// This example shows how to set and retrieve the cooperative level for a
// given device using the TDx_Draw::SetCooperativeLevel() and
// TDx_Draw::TestCooperativeLevel() methods.
//
// NOTE: When setting the cooperative level, DirectX states that you should
// supply the top-most window handle for the entire application, hence
// the use of Application->Handle instead of MainForm->Handle.
//
//---------------------------------------------------------------------------
#ifndef TMainFormH
#define TMainFormH
//---------------------------------------------------------------------------
#include <vcl\Classes.hpp>
#include <vcl\Controls.hpp>
#include <vcl\StdCtrls.hpp>
#include <vcl\Forms.hpp>
#include <vcl\Buttons.hpp>
#include <vcl\ExtCtrls.hpp>
#include <ComCtrls.hpp>
//---------------------------------------------------------------------------
#include "TDx_Draw.h"
#include "TDx_DrawSurface.h"
#include "TDDCaps.h"
#include "TDDPixelFormat.h"
#include "TDDSCaps.h"
#include "TDDSurfaceDesc.h"
//---------------------------------------------------------------------------

#if (__BORLANDC__ < 0x0530) // BCB Version 1
class TMainForm : public TForm
{
#else // BCB Version 3 +
class PACKAGE TMainForm : public TForm
{
#endif

//---------------------------------------------------------------------------
// Anything that is to be displayed within the Object Inspector
//---------------------------------------------------------------------------
__published:

    TPanel *Panel1;
    TPanel *Panel2;
    TPanel *Panel3;
    TPanel *Panel4;
    TPanel *Panel5;
    TLabel *Label2;
    TLabel *Label3;
    TLabel *Label5;
    TLabel *Label6;
    TLabel *ActiveDeviceLabel;

    TMemo *InfoMemo;
    TMemo *StatusMemo;

    TBitBtn *CreateDefaultButton;
    TBitBtn *FullscreenButton;
    TBitBtn *DestroyDefaultButton;
    TBitBtn *ExitButton;
    TBitBtn *ClearButton;
    TDx_Draw *Dx_Draw1;
    TBitBtn *NormalModeButton;
    TLabel *CurrentCoopLevelLabel;
    TLabel *Label7;
    TBitBtn *TestFullscreenButton;

    void __fastcall FormCreate(TObject *Sender);
    void __fastcall FormDestroy(TObject *Sender);

    void __fastcall CreateDefaultButtonClick(TObject *Sender);
    void __fastcall FullscreenButtonClick(TObject *Sender);
    void __fastcall TestFullscreenButtonClick(TObject *Sender);
    void __fastcall NormalModeButtonClick(TObject *Sender);
    void __fastcall DestroyDefaultButtonClick(TObject *Sender);

    void __fastcall ExitButtonClick(TObject *Sender);
    void __fastcall ClearButtonClick(TObject *Sender);

    void __fastcall Dx_Draw1Create(TObject *Sender);
    void __fastcall Dx_Draw1Destroy(TObject *Sender);
    void __fastcall Dx_Draw1Error(TObject *Sender, AnsiString Function,
    AnsiString Error, AnsiString ErrorMessage);

//---------------------------------------------------------------------------
// Anything available to other classes and/or code
//---------------------------------------------------------------------------
public:

    __fastcall TMainForm(TComponent* Owner);
    virtual __fastcall ~TMainForm();

//---------------------------------------------------------------------------
// Anything available only to classes descended from this class
//---------------------------------------------------------------------------
protected:

//---------------------------------------------------------------------------
// Private data not accessible except from within this class
//---------------------------------------------------------------------------
private:

};
//---------------------------------------------------------------------------
#if (__BORLANDC__ < 0x0530) // BCB Version 1
extern TMainForm *MainForm;
#else // BCB Version 3 +
extern PACKAGE TMainForm *MainForm;
#endif
//---------------------------------------------------------------------------
#endif

