//---------------------------------------------------------------------------
// File: TMainForm.cpp, TDx_Draw_Library, Example5 - Display Modes
//
// Date: 1st February, 2002
// Author: Darren John Dwyer
// Copyright: (c) 2002 Darren John Dwyer, All Rights Reserved.
//
// Desciption:
//
// This example shows how to Enumerate all available display modes for the
// default device using the TDx_Draw::EnumDisplayModes() method and the
// TDx_Draw::OnEnumDisplayModes() event.
//
// Once enumerated it allows you to set any of the available display modes
// using the TDx_Draw::SetDisplayMode() method.
//
// Finally, it resets the original display mode (at application startup)
// using TDx_Draw::RestoreDisplayMode().
//
// NOTE: It also contains code to determine the 'bits_per_pixel' of the display
// mode based on the mode's surface description.
//
//---------------------------------------------------------------------------
#include <vcl\vcl.h>
#pragma hdrstop
//---------------------------------------------------------------------------
#include "TMainForm.h"
//---------------------------------------------------------------------------
#if (__BORLANDC__ >= 0x0530) // BCB Version 3 +
    #pragma package(smart_init)
#endif
//---------------------------------------------------------------------------
#pragma resource "*.dfm"
//---------------------------------------------------------------------------
#pragma link "TDx_Draw"
#pragma link "TDx_DrawSurface"
#pragma link "TDDCaps"
#pragma link "TDDPixelFormat"
#pragma link "TDDSCaps"
#pragma link "TDDSurfaceDesc"
#pragma link "TDx_Draw_Library_Install"
//---------------------------------------------------------------------------
TMainForm *MainForm;
//---------------------------------------------------------------------------
AnsiString IntToHexStr( unsigned int hexvalue )
{
char* hexchars[16] = {"0","1","2","3","4","5","6","7","8","9","A","B","C","D","E","F"};
AnsiString string = "0x";
int value = 0x10000000;
for (int i=0;i<8;i++)
    {
    string += hexchars[hexvalue/value];
    hexvalue -= value * (hexvalue/value);
    value = value >> 4;
    }
return string;
}
//---------------------------------------------------------------------------
#define Msg(A)  {\
                if (StatusMemo->Text.Length()>30000) StatusMemo->Text = "";\
                StatusMemo->Lines->Add(A);\
                }
//---------------------------------------------------------------------------
__fastcall TMainForm::TMainForm(TComponent* Owner)
    : TForm(Owner)
{
}
//---------------------------------------------------------------------------
__fastcall TMainForm::~TMainForm()
{
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::FormCreate(TObject *Sender)
{
Width=780;
Height=580;
Position=poScreenCenter;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::FormDestroy(TObject *Sender)
{
if (Dx_Draw1->Created) Dx_Draw1->Destroy();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::CreateDefaultButtonClick(TObject *Sender)
{
Msg("Creating Default Device");
Dx_Draw1->Create(NULL);
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::RestoreModeButtonClick(TObject *Sender)
{
Msg("Restoring Display Mode");
if (Dx_Draw1->RestoreDisplayMode())
    {
    Width=780;
    Height=580;
    Position=poScreenCenter;
    Invalidate();
    }
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::DestroyDefaultButtonClick(TObject *Sender)
{
Msg("Destroying Default Device");
Dx_Draw1->Destroy();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::ExitButtonClick(TObject *Sender)
{
Close();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::ClearButtonClick(TObject *Sender)
{
StatusMemo->Lines->Clear();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Dx_Draw1Create(TObject *Sender)
{
Msg("Device Created: 'Default DirectDraw Device'");
Msg("Setting Fullscreen/Exclusive Mode");
if (Dx_Draw1->SetCooperativeLevel( Application->Handle, DDSCL_FULLSCREEN | DDSCL_EXCLUSIVE ))
    Msg("Fullscreen/Exclusive Mode has been set");
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Dx_Draw1Destroy(TObject *Sender)
{
Msg("Setting Normal Mode");
if (Dx_Draw1->SetCooperativeLevel( Application->Handle, DDSCL_NORMAL ))
    Msg("Normal Mode has been set");
Msg("Device Destroyed: 'Default DirectDraw Device'");
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Dx_Draw1Error(TObject *Sender, AnsiString Function,
    AnsiString Error, AnsiString ErrorMessage)
{
Msg("ERROR ["+Error+"]: "+Function+", "+ErrorMessage);
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::Dx_Draw1EnumDisplayModes(TObject *Sender,
      TDDSurfaceDesc *SurfaceDesc, void *Context, bool &Finished)
{
int width = SurfaceDesc->Width;
int height = SurfaceDesc->Height;
int refresh_rate = SurfaceDesc->RefreshRate;
int bit_depth;

if (SurfaceDesc->PixelFormat->Flags[0] & DDPF_PALETTEINDEXED1) bit_depth = 1;
else if (SurfaceDesc->PixelFormat->Flags[0] & DDPF_PALETTEINDEXED2) bit_depth = 2;
else if (SurfaceDesc->PixelFormat->Flags[0] & DDPF_PALETTEINDEXED4) bit_depth = 4;
else if (SurfaceDesc->PixelFormat->Flags[0] & DDPF_PALETTEINDEXED8) bit_depth = 8;
else if (SurfaceDesc->PixelFormat->Flags[0] & DDPF_PALETTEINDEXEDTO8) bit_depth = 8;
else if (SurfaceDesc->PixelFormat->Flags[0] & DDPF_ALPHA) bit_depth = SurfaceDesc->PixelFormat->AlphaBitDepth[0];
else if (SurfaceDesc->PixelFormat->Flags[0] & DDPF_ALPHAPIXELS) bit_depth = SurfaceDesc->PixelFormat->AlphaBitDepth[0];
else if (SurfaceDesc->PixelFormat->Flags[0] & DDPF_ALPHAPREMULT) bit_depth = SurfaceDesc->PixelFormat->AlphaBitDepth[0];
else if (SurfaceDesc->PixelFormat->Flags[0] & DDPF_BUMPDUDV) bit_depth = SurfaceDesc->PixelFormat->BumpBitCount[0];
else if (SurfaceDesc->PixelFormat->Flags[0] & DDPF_BUMPLUMINANCE) bit_depth = SurfaceDesc->PixelFormat->BumpBitCount[0];
else if (SurfaceDesc->PixelFormat->Flags[0] & DDPF_RGB) bit_depth = SurfaceDesc->PixelFormat->RGBBitCount[0];
else if (SurfaceDesc->PixelFormat->Flags[0] & DDPF_YUV) bit_depth = SurfaceDesc->PixelFormat->YUVBitCount[0];

if (NumModes<MAX_MODES)
    {
    Widths[NumModes] = width;
    Heights[NumModes] = height;
    BitDepths[NumModes] = bit_depth;
    RefreshRates[NumModes] = refresh_rate;
    NumModes++;

    ModesListBox->Items->Add( IntToStr(width)+"x"+IntToStr(height)+"x"+IntToStr(bit_depth)+" @ "+IntToStr(refresh_rate)+"Hz" );
    }
else
    Finished = true;
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::EnumModesButtonClick(TObject *Sender)
{
Msg("Enumerating All Display Modes");
NumModes=0;
ModesListBox->Items->Clear();
Dx_Draw1->EnumDisplayModes(0,NULL,NULL);
if (NumModes==0)
    {
    Msg("Did not find any Display Modes");
    ModesListBox->Items->Add( "-- none --" );
    }
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::SetModeButtonClick(TObject *Sender)
{
if (ModesListBox->Items->Count>0)
    {
    ShowMessage("Please Note:\n\
You may experience difficulty restoring your display mode.\n\
To return to the original display mode at any time, press ALT-R (Restore) or ALT-X (eXit)" );

    int index = ModesListBox->ItemIndex;
    Dx_Draw1->SetDisplayMode( Widths[index], Heights[index], BitDepths[index], RefreshRates[index], 0 );

    Top=0;
    Left=0;
    Width=780;
    Height=580;
    Invalidate();
    }
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::NextModeButtonClick(TObject *Sender)
{
if (ModesListBox->Items->Count>0)
    {
    Msg("Selecting Next Available Mode");
    if (ModesListBox->ItemIndex==ModesListBox->Items->Count-1) ModesListBox->ItemIndex=0;
    else ModesListBox->ItemIndex = ModesListBox->ItemIndex + 1;
    ModesListBox->OnClick(this);
    }
else
    ModesListBox->ItemIndex = -1;
}
//---------------------------------------------------------------------------

void __fastcall TMainForm::ModesListBoxClick(TObject *Sender)
{
if (ModesListBox->ItemIndex!=-1)
    {
    int index = ModesListBox->ItemIndex;
    AnsiString name = ModesListBox->Items->Strings[index];

    Msg("Mode Selected: '"+name+"'");
    CurrentModeLabel->Caption = name;
    }
else
    CurrentModeLabel->Caption = "-- default --";
}
//---------------------------------------------------------------------------

