//---------------------------------------------------------------------------
// File: TMainForm.h
//
//  TDx_Draw_Library
//  ExampleQ - Converted DirectX Code - Donuts
//
// Date: 1st February, 2002
// Author: Darren John Dwyer
// Copyright: (c) 2002 Darren John Dwyer, All Rights Reserved.
//
// Description:
//
// This example is a fairly straightforward conversion from
// DirectDraw's Example4 Demo.
//
// It uses the file "all.bmp" supplied with the DirectX SDK.
//
// This application has been "translated" from the Original DirectX Code.
// The code is very similar to the "Example4" code supplied by MS, just
// converted so it works with the TDx_Draw_Library and C++ Builder.
//
// It consists of the following code sections :-
//
// FormCreate() - sets up DirectDraw, loads the images, sets palettes, etc.
// FormDestroy() - cleans up.
// FormKeyDown() - handles keystrokes.
// FormMouseMove() - updates the mouse cursor display
// FormResize() - updates the size of the status list box (when errors occur)
//
// OnMinimize() - stops animation
// OnRestore() - restores the cooperative level and continues animation
// OnIdle() - calls a method called UpdateFrame() to paint each frame.
//
// UpdateFrame() - paints each frame.
//
// PrimarySurface::SurfaceLost() - handles restoration of surface data.
// Dx_Draw1::OnError() - handles all DirectDraw errors.
// Dx_Draw1::OnEnumDisplayModes() - determines available modes.
//
//---------------------------------------------------------------------------
// SPECIAL NOTE:
//
// This application uses 640x480x8 normally.
//
// In order to save a snapshot of the screen, I've modified it slightly
// so that the snapshot palette is saved too (using 640x480x32).
//
// When doing this, note that you don't actually need to interface with
// DirectDraw palette functions. A nice bonus.
//
// To switch between the two, recompile the code after changing 'BITS'
// ( start of .CPP file, after the comment block )
//
// When 'BITS' is 8, no palette information appears to be saved by BCB.
// When 'BITS' is 32, the palette information is saved correctly by BCB.
//
//---------------------------------------------------------------------------
#ifndef TMainFormH
#define TMainFormH
//---------------------------------------------------------------------------
#include <Classes.hpp>
#include <Controls.hpp>
#include <StdCtrls.hpp>
#include <Forms.hpp>
//---------------------------------------------------------------------------
#include "TDx_Draw.h"
#include "TDx_DrawSurface.h"
#include "TDDColorKey.h"
#include "TDDSCaps.h"
#include "TDDSurfaceDesc.h"
#include "TDx_DrawPalette.h"
#include <Buttons.hpp>
#include "TDDCaps.h"
#include <ExtCtrls.hpp>
#include "tddcolorkey.h"
#include "tddscaps.h"
#include "tddsurfacedesc.h"
#include "tdx_draw.h"
#include "tdx_drawpalette.h"
#include "tdx_drawsurface.h"
//---------------------------------------------------------------------------
class TMainForm : public TForm
{
__published:    // IDE-managed Components
    TDx_Draw *Dx_Draw1;
    TDx_DrawSurface *PrimarySurface;
    TDx_DrawSurface *TempSurface;
    TDx_DrawSurface *BackBuffer;
    TDDColorKey *PrimarySurfaceColorKey;
    TDDSurfaceDesc *PrimarySurfaceDesc;
    TDDSCaps *PrimarySurfaceCaps;
    TDDSCaps *BackBufferCaps;
    TDx_DrawPalette *PrimarySurfacePalette;
    TLabel *Label1;
    TListBox *StatusListBox;
    TLabel *Label2;
    void __fastcall FormKeyDown(TObject *Sender, WORD &Key,
          TShiftState Shift);
    void __fastcall PrimarySurfaceSurfaceLost(TObject *Sender);
    void __fastcall FormMouseMove(TObject *Sender, TShiftState Shift,
          int X, int Y);
    void __fastcall FormCreate(TObject *Sender);
    void __fastcall FormDestroy(TObject *Sender);
    void __fastcall Dx_Draw1Error(TObject *Sender, AnsiString TheFunction,
          AnsiString TheError, AnsiString TheErrorMessage);
    void __fastcall FormResize(TObject *Sender);
    void __fastcall Dx_Draw1EnumDisplayModes(TObject *Sender,
          TDDSurfaceDesc *SurfaceDesc, void *Context, bool &Finished);
    
private:    // User declarations

    bool    ModeExists;
    bool    Active;
    int     NumDisplayModes;
    int     FrameRate;
    int     FrameCount;
    int     LastFrameCount;
    int     FrameTime;
    int     LastFrameTime;
    int     LastMouseX;
    int     LastMouseY;

    void __fastcall OnIdle(TObject *Sender, bool &Done);
    void __fastcall UpdateFrame();

    void __fastcall OnMinimize( TObject* Sender );
    void __fastcall OnRestore( TObject* Sender );

public:        // User declarations
    __fastcall TMainForm(TComponent* Owner);

};
//---------------------------------------------------------------------------
extern TMainForm *MainForm;
//---------------------------------------------------------------------------
#endif
